package mn.geoProcessing.corte;

import java.util.Map;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.ILayerDefinition;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.StrategyManager;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.geoprocess.core.fmap.AbstractGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.fmap.DefinitionUtils;
import com.iver.cit.gvsig.geoprocess.core.fmap.FeaturePersisterProcessor2;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessException;
import com.iver.cit.gvsig.geoprocess.core.fmap.IOverlayGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.fmap.XTypes;
import com.iver.utiles.swing.threads.CancellableMonitorable;
import com.iver.utiles.swing.threads.DefaultCancellableMonitorable;
import com.iver.utiles.swing.threads.IMonitorableTask;
/**
 * Computes corte between two layers.
 * 
 * @author azabala
 *
 */
public class CorteGeoprocess extends AbstractGeoprocess
								implements IOverlayGeoprocess {
	
	/**
	 * overlay layer
	 */
	private FLyrVect overlayLayer;

	/**
	 * Schema of the result layer
	 */
	private ILayerDefinition resultLayerDefinition;

	/**
	 * flag to only clip selection of input layer
	 */
	private boolean onlyFirstLayerSelection = false;

	/**
	 * flag to only clip with selection of clipping layer
	 */
	private boolean onlyClipLayerSelection = false;
	
	public CorteGeoprocess(FLyrVect inputLayer){
		setFirstOperand(inputLayer);
	}
	
	public void setSecondOperand(FLyrVect overlayLayer) {
		this.overlayLayer = overlayLayer;

	}

	public void setFirstOperand(FLyrVect firstLayer) {
		this.firstLayer = firstLayer;
	}

	public void setParameters(Map params) throws GeoprocessException {
		Boolean firstLayerSelection = (Boolean) params
				.get("firstlayerselection");
		if (firstLayerSelection != null)
			this.onlyFirstLayerSelection = firstLayerSelection.booleanValue();

		Boolean secondLayerSelection = (Boolean) params
				.get("secondlayerselection");
		if (secondLayerSelection != null)
			this.onlyClipLayerSelection = secondLayerSelection.booleanValue();

	}

	public void checkPreconditions() throws GeoprocessException {
		if (firstLayer == null)
			throw new GeoprocessException("Interseccion: capa de entrada a null");
		if (overlayLayer == null)
			throw new GeoprocessException("Interseccion: capa de union a null");
		if (this.writer == null || this.schemaManager == null) {
			throw new GeoprocessException(
					"Operacion de interseccion sin especificar capa de resultados");
		}
		try {
			if ((firstLayer.getShapeType() != XTypes.POLYGON) &&
(firstLayer.getShapeType() != XTypes.MULTI)) {
//FIXME De momento admito MULTI para pruebas, pero esto
//hace mas complejo TODO
				throw new GeoprocessException(
						"Primera capa de interseccion no es de polgonos");
			}
			if ((overlayLayer.getShapeType() != XTypes.POLYGON) && 
(overlayLayer.getShapeType() != XTypes.MULTI))  {
				throw new GeoprocessException(
						"Segunda capa de interseccion no es de polgonos");
			}
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			throw new GeoprocessException(
					"Error al tratar de chequear si las capas a intersectar son de polgonos");
		}

	}

public void process() throws GeoprocessException {
	
		try {
			//Prepare the result
			this.schemaManager.createSchema(createLayerDefinition());
			writer.preProcess();
			Strategy strategy = 
				StrategyManager.getStrategy(firstLayer);
			Strategy overlayStrategy = 
				StrategyManager.getStrategy(overlayLayer);
			FeaturePersisterProcessor2 featureProcessor =
				new FeaturePersisterProcessor2(writer);
			CorteVisitor visitor = new CorteVisitor(overlayLayer, 
														featureProcessor, 
														overlayStrategy,
														onlyClipLayerSelection);
			if(this.onlyFirstLayerSelection){
				strategy.process(visitor, 
					this.firstLayer.getRecordset().getSelection());
			}else{
				strategy.process(visitor);
			}
			
		} catch (EditionException e) {
			throw new GeoprocessException("Error al crear el esquema/fichero de la nueva capa");
		} catch (DriverException e) {
			throw new GeoprocessException("Error de driver al calcular el geoproceso interseccion");
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			throw new GeoprocessException("Error de driver al calcular el geoproceso interseccion");
		} catch (VisitException e) {
			throw new GeoprocessException("Error al procesar el feature de una capa durante el geoproceso interseccion");
		}
	}	


	public void cancel() {
		try {
			schemaManager.removeSchema("");
		} catch (EditionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public ILayerDefinition createLayerDefinition() {
		if (resultLayerDefinition == null) {
			try {
				resultLayerDefinition = DefinitionUtils.mergeLayerDefinitions(
						firstLayer, overlayLayer);
			} catch (Exception e) {
				// TODO Quizas createLayerDefinition deberia lanzar
				// una excepcion
				e.printStackTrace();
			}
		}
		return resultLayerDefinition;
	}


	public IMonitorableTask createTask() {
		try {
			return new CorteMonitorableTask();
		} catch (Exception e) {
			return null;
		}
	}
	
	/**
	 * IMonitorableTask that allows to run intersection geoprocess in background,
	 * with cancelation requests.
	 * 
	 * @author azabala
	 * 
	 */
	class CorteMonitorableTask implements IMonitorableTask {
		private CancellableMonitorable cancelMonitor = null;
		String INTERSECTION_MESSAGE = PluginServices.getText(this, "Mensaje_corte");
		String INTERSECTION_NOTE = PluginServices.getText(this, "Mensaje_procesando_corte");
		String OF = PluginServices.getText(this, "De");
		private boolean finished = false;

		CorteMonitorableTask() throws DriverIOException, com.iver.cit.gvsig.fmap.DriverException {
			initialize();
		}
		void initialize() throws DriverIOException, com.iver.cit.gvsig.fmap.DriverException {
			cancelMonitor = createCancelMonitor();
		}

		private CancellableMonitorable createCancelMonitor()
				throws DriverIOException, com.iver.cit.gvsig.fmap.DriverException {
			DefaultCancellableMonitorable monitor = new 
							DefaultCancellableMonitorable();
			monitor.setInitialStep(0);
			//Really its undeterminated, but  we must to process all
			//elements of first layer (or selection) we are going to 
			//consideer determinated
			monitor.setDeterminatedProcess(true);
			int numSteps = 0;
			if (onlyFirstLayerSelection) {
				FBitSet selection = firstLayer.getRecordset().getSelection();
				numSteps = selection.cardinality();
			} else {
				numSteps = firstLayer.getSource().getShapeCount();
			}
			monitor.setFinalStep(numSteps);
			return monitor;
		}

		public int getInitialStep() {
			return cancelMonitor.getInitialStep();
		}

		public int getFinishStep() {
			return cancelMonitor.getFinalStep();
		}

		public int getCurrentStep() {
			return cancelMonitor.getCurrentStep();
		}

		public String getStatusMessage() {
			return INTERSECTION_MESSAGE;
		}

		public String getNote() {
			return INTERSECTION_NOTE + " " +
			getCurrentStep() + " " +
			OF + " "
			+ getFinishStep();
		}

		public void cancel() {
			((DefaultCancellableMonitorable) cancelMonitor).setCanceled(true);
			//This does this geoprocess athomic. In this
			//call we remove result files
			CorteGeoprocess.this.cancel();
		}

		public void run() throws GeoprocessException {

			try {
				//Prepare the result
				schemaManager.createSchema(createLayerDefinition());
				writer.preProcess();
				Strategy strategy = 
					StrategyManager.getStrategy(firstLayer);
				Strategy overlayStrategy = 
					StrategyManager.getStrategy(overlayLayer);
				FeaturePersisterProcessor2 featureProcessor =
					new FeaturePersisterProcessor2(writer);
				CorteVisitor visitor = new CorteVisitor(overlayLayer, 
															featureProcessor, 
															overlayStrategy,
															onlyClipLayerSelection);
//				AwtIntersectVisitor visitor = new AwtIntersectVisitor(overlayLayer, 
//						featureProcessor, 
//						overlayStrategy,
//						onlyClipLayerSelection);
				if(onlyFirstLayerSelection){
					strategy.process(visitor, 
						firstLayer.getRecordset().getSelection(),
						cancelMonitor);
				}else{
					strategy.process(visitor, cancelMonitor);
				}
				
			} catch (EditionException e) {
				throw new GeoprocessException("Error al crear el esquema/fichero de la nueva capa");
			} catch (DriverException e) {
				throw new GeoprocessException("Error de driver al calcular el geoproceso interseccion");
			} catch (com.iver.cit.gvsig.fmap.DriverException e) {
				throw new GeoprocessException("Error de driver al calcular el geoproceso interseccion");
			} catch (VisitException e) {
				throw new GeoprocessException("Error al procesar el feature de una capa durante el geoproceso interseccion");
			}finally{
				finished = true;
			}
		}

		public boolean isDefined() {
			return cancelMonitor.isDeterminatedProcess();
		}

		public boolean isCanceled() {
			return cancelMonitor.isCanceled();
		}

		public boolean isFinished() {
			return finished;
		}
	}

}
