package mn.geoProcessing.corte;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.drivers.SHPLayerDefinition;
import com.iver.cit.gvsig.fmap.edition.IWriter;
import com.iver.cit.gvsig.fmap.edition.ShpSchemaManager;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.geoprocess.core.fmap.AbstractGeoprocessController;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessException;
import com.iver.cit.gvsig.geoprocess.core.fmap.IGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.gui.AddResultLayerTask;
import com.iver.cit.gvsig.geoprocess.core.gui.IGeoprocessPanel;
import com.iver.cit.gvsig.geoprocess.core.gui.OverlayPanelIF;
import com.iver.utiles.swing.threads.IMonitorableTask;
import com.iver.utiles.swing.threads.MonitorableDecoratorMainFirst;

public class CorteGeoprocessController extends
		AbstractGeoprocessController {


	private OverlayPanelIF corteGeoProcessingPanel;
	private CorteGeoprocess corte;

	public void setView(IGeoprocessPanel viewPanel) {
		this.corteGeoProcessingPanel =
			(OverlayPanelIF) viewPanel;
	}

	public IGeoprocess getGeoprocess() {
		return corte;
	}

	public boolean launchGeoprocess() {
		FLyrVect inputLayer = corteGeoProcessingPanel.getInputLayer();
		FLayers layers = corteGeoProcessingPanel.getFLayers();
		FLyrVect overlayLayer = corteGeoProcessingPanel.getSecondLayer();
		File outputFile = null;
		try {
			outputFile = corteGeoProcessingPanel.getOutputFile();
		} catch (FileNotFoundException e3) {
			String error = PluginServices.getText(this, "Error_entrada_datos");
			String errorDescription = PluginServices.getText(this, "Error_seleccionar_resultado");
			corteGeoProcessingPanel.error(errorDescription, error);
			return false;
		}
		if (outputFile == null || (outputFile.getAbsolutePath().length() == 0)) {
			String error = PluginServices.getText(this, "Error_entrada_datos");
			String errorDescription = PluginServices.getText(this, "Error_seleccionar_resultado");
			corteGeoProcessingPanel.error(errorDescription, error);
			return false;
		}
		if (outputFile.exists()) {
			if (!corteGeoProcessingPanel.askForOverwriteOutputFile(outputFile)) {
				return false;
			}
		}

		CorteGeoprocess corte = new CorteGeoprocess(
				inputLayer);
		corte.setSecondOperand(overlayLayer);
		SHPLayerDefinition definition = (SHPLayerDefinition) corte
				.createLayerDefinition();
		definition.setFile(outputFile);
		ShpSchemaManager schemaManager = new ShpSchemaManager(outputFile.getAbsolutePath());
		IWriter writer;
		try {
			writer = getShpWriter(definition);
		} catch (Exception e1) {
			String error = PluginServices.getText(this, "Error_escritura_resultados");
			String errorDescription = PluginServices.getText(this, "Error_preparar_escritura_resultados");
			corteGeoProcessingPanel.error(errorDescription, error);
			return false;
		}

		corte.setResultLayerProperties(writer, schemaManager);
		HashMap params = new HashMap();
		boolean onlySelectedFirst = corteGeoProcessingPanel
				.onlyFirstLayerSelected();
		boolean onlySelectedSecond = corteGeoProcessingPanel
				.onlySecondLayerSelected();
		Boolean first = new Boolean(onlySelectedFirst);
		params.put("firstlayerselection", first);

		Boolean second = new Boolean(onlySelectedSecond);
		params.put("secondlayerselection", second);

		try {
			corte.setParameters(params);
			corte.checkPreconditions();
			IMonitorableTask task1 = corte.createTask();
			if(task1 == null){
				return false;
			}
			AddResultLayerTask task2 = new AddResultLayerTask(corte);
			task2.setLayers(layers);
			MonitorableDecoratorMainFirst globalTask = new MonitorableDecoratorMainFirst(task1,
					task2);
			if(!overlayLayer.isSpatiallyIndexed()){
				final IMonitorableTask sptIdxTask =
					corteGeoProcessingPanel.askForSpatialIndexCreation(overlayLayer);
				if(sptIdxTask != null){
					PluginServices.backgroundExecution(
							new Runnable(){
						public void run() {
							PluginServices.
							cancelableBackgroundExecution(sptIdxTask);
						}}
					);
				}
			}//if
			if (globalTask.preprocess())
				PluginServices.cancelableBackgroundExecution(globalTask);
		} catch (GeoprocessException e) {
			String error = PluginServices.getText(this, "Error_ejecucion");
			String errorDescription = PluginServices.getText(this, "Error_fallo_geoproceso");
			errorDescription = "<html>" + errorDescription + ":<br>" + e.getMessage()+ "</html>";
			corteGeoProcessingPanel.error(errorDescription, error);
			return false;
		}
		return true;
	}
	public int getWidth() {
		return 700;
	}

	public int getHeight() {
		return 300;
	}
}

