package mn.geoProcessing.corte;

import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.iver.cit.gvsig.fmap.drivers.ILayerDefinition;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.fmap.layers.layerOperations.VectorialData;
import com.iver.cit.gvsig.fmap.operations.strategies.FeatureVisitor;
import com.iver.cit.gvsig.fmap.operations.strategies.Strategy;
import com.iver.cit.gvsig.fmap.operations.strategies.VisitException;
import com.iver.cit.gvsig.geoprocess.core.fmap.FeatureFactory;
import com.iver.cit.gvsig.geoprocess.core.fmap.FeatureProcessor;
import com.iver.cit.gvsig.geoprocess.core.fmap.XTypes;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.MultiPolygon;
import com.vividsolutions.jts.geom.Polygon;
import com.vividsolutions.jts.precision.EnhancedPrecisionOp;

/**
 * Computes intersections of features of a layer with features of an overlay
 * layer.
 * 
 * @author azabala
 * 
 * FIXME Crear una clase abstracta: OverlayVisitor, pues todos los visitors de
 * un overlay son muy parecidos.
 * 
 */
public class CorteVisitor implements FeatureVisitor {

	/**
	 * Allows to get attributes of first layer features which is being
	 * intersected
	 */
	SelectableDataSource firstRs;

	/**
	 * Number of fields of first layer recordset
	 */
	int numFieldsA;

	/**
	 * looks for overlay features of the processed feauture by spatial criteria
	 * (queryByEnvelope)
	 */
	FLyrVect overlayLayer;

	/**
	 * Strategy to process overlay layer
	 */
	Strategy strategy;

	/**
	 * flag to decide if process onlye features selected in overlay layer.
	 */
	boolean onlyOverlaySelection;

	/**
	 * Gets attributes of second layer features which are being intersected.
	 */
	SelectableDataSource secondRs;

	/**
	 * Number of fields of second layer recordset
	 */
	int numFieldsB;

	/**
	 * Schema of the result layer
	 */
	ILayerDefinition layerDefinition;

	/**
	 * It processes features resulting of intersetions. It could saves them in
	 * persistent datastore, caching them, reprocess them, reprojects them, etc.
	 */
	FeatureProcessor featureProcessor;
	
	/**
	 * Constructor
	 * @param overlayLayer
	 * @param processor
	 * @param strategy
	 * @param onlyOverlaySelection
	 * @throws DriverException
	 * @throws com.iver.cit.gvsig.fmap.DriverException
	 */
	public CorteVisitor(FLyrVect overlayLayer, FeatureProcessor processor,
			Strategy strategy, boolean onlyOverlaySelection)
			throws DriverException, com.iver.cit.gvsig.fmap.DriverException {
		this.overlayLayer = overlayLayer;
		this.featureProcessor = processor;
		this.strategy = strategy;
		this.onlyOverlaySelection = onlyOverlaySelection;
		secondRs = overlayLayer.getRecordset();
		numFieldsB = secondRs.getFieldCount();
	}

	public void visit(IGeometry g, final int index) throws VisitException {
		if(g == null)
			return;
		if(g.getGeometryType() != XTypes.POLYGON &&
				g.getGeometryType() != XTypes.MULTI)
			return;
		final Geometry firstJts = g.toJTSGeometry();
		final boolean onlyOverlay = onlyOverlaySelection;
		try {
			strategy.process(new FeatureVisitor(){
				public void visit(IGeometry g, int indexOverlay) throws VisitException {
					if(onlyOverlay){
						try {
							if(!overlayLayer.getRecordset().getSelection().get(index))
								return;
						} catch (com.iver.cit.gvsig.fmap.DriverException e) {
							throw new VisitException(
							"Error en interseccion: verificando si un posible overlay esta seleccionado", e);
						}//geometry g is not selected
					}	
					if(g.getGeometryType() != XTypes.POLYGON &&
							g.getGeometryType() != XTypes.MULTI)
						return;
					Geometry overlayJts = g.toJTSGeometry();					
					if (firstJts.intersects(overlayJts)) {
						Geometry newGeoJts = EnhancedPrecisionOp.intersection(firstJts, overlayJts);
						if (!(newGeoJts instanceof Polygon)
								&& !(newGeoJts instanceof MultiPolygon)) {
							// intersection of adjacent polygons is a linestring
							// but we are not interested in it
							return;
						}
						IFeature intersectionFeature;
						try {
							intersectionFeature = createFeature(newGeoJts,
									index, indexOverlay);
						} catch (DriverException e) {
							throw new VisitException(
									"Error al crear el feature resultante de la interseccion");
						}
						featureProcessor.processFeature(intersectionFeature);
					}// if intersects	
				}

				public String getProcessDescription() {
					return "Computing intersections of a polygon with its adjacents";
				}
				public void stop(FLayer layer) {
				}
				public boolean start(FLayer layer) {
					return true;
				}},g.getBounds2D());
			
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			throw new VisitException(
					"Error buscando los overlays que intersectan con un feature");
		} 
	}

	public void stop(FLayer layer) {
		featureProcessor.finish();
	}

	public boolean start(FLayer layer) {
		if (layer instanceof AlphanumericData && layer instanceof VectorialData) {
			try {
				this.firstRs = ((AlphanumericData) layer).getRecordset();
				numFieldsA = firstRs.getFieldCount();
				this.featureProcessor.start();
			} catch (com.iver.cit.gvsig.fmap.DriverException e) {
				return false;
			} catch (DriverException e) {
				return false;
			} catch (EditionException e) {
				return false;
			}
			return true;
		}
		return false;
	}
	

	/**
	 * TODO Meter este metodo en FeatureFactory, porque lo estoy copypasteando
	 * en todos los Visitor
	 * 
	 * FIXME El esquema es resultado de la mezcla de los esquemas de dos capas
	 * Como saber qu campo de ILayerDefinition se toma de firstRs y cual de secondRs?
	 * Mirando DefinitionUtils, vemos que primero se recorre firstLayer,
	 * y luego secondLayer. REVISAR
	 */
	private IFeature createFeature(Geometry jtsGeometry, int firstLayerIndex,
			int overlayLayerIndex) throws DriverException {
		IFeature solution = null;
		IGeometry intersectGeometry = FConverter.jts_to_igeometry(jtsGeometry);		
		
		Value[] featureAttr = new Value[numFieldsA + numFieldsB];
		for (int indexField = 0; indexField < numFieldsA; indexField++) {
			featureAttr[indexField] = firstRs.getFieldValue(firstLayerIndex,
					indexField);
		}
		for (int indexFieldB = 0; indexFieldB < numFieldsB; indexFieldB++) {
			featureAttr[numFieldsA + indexFieldB] = secondRs.getFieldValue(
					overlayLayerIndex, indexFieldB);
		}
		solution = FeatureFactory.createFeature(featureAttr, intersectGeometry);
		return solution;
		
	}

	public void setFeatureProcessor(FeatureProcessor featureProcessor) {
		this.featureProcessor = featureProcessor;
	}

	public String getProcessDescription() {
		return "Computing intersections between two layers";
	}

	public ILayerDefinition getLayerDefinition() {
		return layerDefinition;
	}

	public void setLayerDefinition(ILayerDefinition layerDefinition) {
		this.layerDefinition = layerDefinition;
	}

}
