package mn.geoProcessing.area;

import java.sql.Types;
import java.util.HashMap;
import java.util.Map;

import javax.swing.SwingUtilities;



import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.cit.gvsig.EditionUtilities;
import com.iver.cit.gvsig.StartEditing;
import com.iver.cit.gvsig.StopEditing;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.IRow;
import com.iver.cit.gvsig.fmap.drivers.ConcreteMemoryDriver;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.drivers.ILayerDefinition;
import com.iver.cit.gvsig.fmap.edition.EditionEvent;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.cit.gvsig.fmap.edition.ISpatialWriter;
import com.iver.cit.gvsig.fmap.edition.VectorialEditableAdapter;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLyrVect;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.ReadableVectorial;
import com.iver.cit.gvsig.geoprocess.core.fmap.AbstractGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.fmap.GeoprocessException;
import com.iver.cit.gvsig.geoprocess.core.fmap.IOneLayerGeoprocess;
import com.iver.cit.gvsig.geoprocess.core.fmap.XTypes;
import com.iver.cit.gvsig.project.documents.table.gui.Table;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.cit.gvsig.project.documents.view.legend.CreateSpatialIndexMonitorableTask;
import com.iver.utiles.swing.threads.CancellableMonitorable;
import com.iver.utiles.swing.threads.DefaultCancellableMonitorable;
import com.iver.utiles.swing.threads.IMonitorableTask;
import com.vividsolutions.jts.geom.Geometry;

/**
 * Calculo de area/permetro/longitud
 * 
 * Calcula el area/permetro/longitud y la introduce en un campo de:
 * o de la misma capa
 * o en una capa nueva
 *  
 * @author eduardo
 * 
 */
public class AreaGeoprocess extends AbstractGeoprocess implements
	IOneLayerGeoprocess {

    private AreaUnidades moUnidades;
	private View moVista;
	
    private String msCampoArea;
    private String msCampoPerimetro;
    private boolean mbCalculoArea;
    private boolean mbCalculoPerimetro;
    private String msNombreCapa;
    private boolean mbGuardar;
    private String msUnidadArea;
    private String msUnidadPerimetro;
    private int mlNumeroDecimales = 4;
    

	/**
	 * Constructor
	 * 
	 */
	public AreaGeoprocess(FLyrVect inputLayer, View poVista) {
		moVista = poVista;
		setFirstOperand(inputLayer);
		moUnidades = new AreaUnidades();
	}

	public void setFirstOperand(FLyrVect firstLayer) {
		this.firstLayer = firstLayer;
	}
    public void setCalculoArea(boolean pbValor){
        mbCalculoArea = pbValor;
    }
    public void setNombreCapa(String psNombreCapa){
        msNombreCapa = psNombreCapa;
    }
    public void setCalculoPerimetro(boolean pbValor){
        mbCalculoPerimetro = pbValor;
    }
    public void setCampoArea(String psArea){
        msCampoArea = psArea;
    }
    public void setCampoPerimetro(String psPerimetro){
        msCampoPerimetro = psPerimetro;
    }
    public void setGuardar(boolean pbGuardar){
    	mbGuardar = pbGuardar;
    }
    public void setUnidadArea(String psUnidadArea){
    	msUnidadArea = psUnidadArea;
    }
    public void setUnidadPerimetro(String psUnidadPerimetro){
    	msUnidadPerimetro = psUnidadPerimetro;
    }
    public void setNumeroDecimales(int plNumeroDecimales){
    	mlNumeroDecimales = plNumeroDecimales;
    }

	public void setParameters(Map params) throws GeoprocessException {
	}

	public void checkPreconditions() throws GeoprocessException {
		if (firstLayer == null) {
			throw new GeoprocessException("Clculo Area: capa de entrada a null");
		}
		try {
			if (firstLayer.getShapeType() != XTypes.POLYGON &&
				(firstLayer.getShapeType() != XTypes.MULTI) && 
					mbCalculoArea) {
					throw new GeoprocessException(
							"La capa para el clculo de rea no es de polgonos");
			}
			if (firstLayer.getShapeType() != XTypes.POLYGON &&
				(firstLayer.getShapeType() != XTypes.MULTI) && 
				(firstLayer.getShapeType() != XTypes.LINE) && 
				mbCalculoPerimetro) {
					throw new GeoprocessException(
							"La capa para el clculo de permetro/longitud no es de polgonos/lneas");
			}
			if(mbCalculoArea && (msCampoArea==null || msCampoArea.trim().equals("")) ){
				throw new GeoprocessException(
				"El cculo de rea no tiene un campo asociado");
			}
			if(mbCalculoPerimetro && (msCampoPerimetro==null || msCampoPerimetro.trim().equals("")) ){
				throw new GeoprocessException(
				"El cculo de permetro/longitud no tiene un campo asociado");
			}
			if(writer == null && !firstLayer.isWritable()){
				throw new GeoprocessException(
					"La capa no es editable");
			}
		} catch (com.iver.cit.gvsig.fmap.DriverException e) {
			throw new GeoprocessException(
					"Error al tratar de chequear si la capa para el cculo es de polgonos/lneas");
		}
	}

	public void process() throws GeoprocessException {
		try{
			createTask().run();
		}catch (Throwable e) {
			throw new GeoprocessException(e);
		}
//		try {
//			if(writer == null){
//				firstLayer.setEditing(true);
//			}
//			int lCampoArea = -1;
//			if(mbCalculoArea){
//				lCampoArea = crearSeleccionarCampo(msCampoArea);
//			}
//			int lCampoPerimetro = -1;
//			if(mbCalculoPerimetro){
//				lCampoPerimetro = crearSeleccionarCampo(msCampoPerimetro);
//			}
//			
//			visitor = new AreaVisitor(resultLayerDefinition, schemaManager, writer);
//			Strategy strategy = StrategyManager.getStrategy(firstLayer);
//			strategy.process(visitor);
//			if(writer == null){
//				firstLayer.setEditing(false);
//			}
//		} catch (Exception e) {
//			e.printStackTrace();
//			throw new GeoprocessException(e);
//		}
		throw new InternalError("No implementado");
			
	}


	
	

	public IMonitorableTask createTask() {
		final CancellableMonitorable cancelMonitor = 
			createCancelMonitor();

		return new IMonitorableTask() {
			String Area_GEOP_MSG = PluginServices.getText(this,
												"Mensaje_Area");
			String Area_MESSAGE = PluginServices.getText(this,
			"Mensaje_procesando_Area_primero");
			String INTERS_MESSAGE = PluginServices.getText(this,
				"Mensaje_procesando_Area_segundo");
			String of = PluginServices.getText(this, "De");
			String currentMsg = Area_MESSAGE;
			private boolean finished = false;

			public int getInitialStep() {
				return cancelMonitor.getInitialStep();
			}

			public int getFinishStep() {
				return cancelMonitor.getFinalStep();
			}

			public int getCurrentStep() {
				return cancelMonitor.getCurrentStep();
			}

			public String getStatusMessage() {
				return Area_GEOP_MSG;
			}

			public String getNote() {
				return currentMsg  + 
				" " +
				getCurrentStep()+ 
				" "+
				of+
				" "+
				getFinishStep();
			}

			public void cancel() {
				((DefaultCancellableMonitorable) cancelMonitor)
						.setCanceled(true);
				AreaGeoprocess.this.cancel();
			}

			public void run() {
				IDatosAGuardar loGuardar = null;
				if(msNombreCapa!=null && !msNombreCapa.equals("")){
					//si es capa en memoria siempre se guarda
					mbGuardar = true;
					loGuardar = new JDatosAGuardarCapaMemoria(firstLayer, msNombreCapa);
				}else{
					loGuardar = new JDatosAGuardarMismaCapa(firstLayer, moVista);
				}
				try {
					ReadableVectorial rv = firstLayer.getSource();
					int lCampoArea = -1;
					if(mbCalculoArea){
						lCampoArea = loGuardar.crearSeleccionarCampo(msCampoArea);
					}
					int lCampoPerimetro = -1;
					if(mbCalculoPerimetro){
						lCampoPerimetro = loGuardar.crearSeleccionarCampo(msCampoPerimetro);
					}
					currentMsg = INTERS_MESSAGE;

					rv.start();
					try{
						for (int i = 0; i < rv.getShapeCount(); i++) {
							//monitor
							if (cancelMonitor != null) {
								cancelMonitor.setCurrentStep(i);
								cancelMonitor.reportStep();
								if (cancelMonitor.isCanceled()){
									throw new Exception("Cancelado");
								}
							}
							//recuperamos la geometria
							IGeometry geom = rv.getShape(i);
							if (geom == null)
								continue;
							//transformatos a jts
							Geometry jtsGeom = geom.toJTSGeometry();
							//recuperamos los valores
							IRow loRow = rv.getFeature(i);
							Value[] loValores = loRow.getAttributes();
							//establecemos los valores
							HashMap loMap = new HashMap(); 
							if(mbCalculoArea){
								loMap.put(
									new Integer(lCampoArea), 
									ValueFactory.createValue(
											moUnidades.getConvertirDeM2(msUnidadArea, jtsGeom.getArea(), mlNumeroDecimales) 
											));
//								loValores[lCampoArea] = 
//									ValueFactory.createValue(jtsGeom.getArea());
							}
							if(mbCalculoPerimetro){
								loMap.put(
									new Integer(lCampoPerimetro), 
									ValueFactory.createValue(
											moUnidades.getConvertirDeM(msUnidadPerimetro, jtsGeom.getLength(), mlNumeroDecimales) 
											));
//								loValores[lCampoPerimetro]=
//									ValueFactory.createValue(jtsGeom.getLength());	
							}
							loGuardar.guardar(i, geom, loValores, loRow.getID(), loMap);
						}// for
						if(mbGuardar){
							loGuardar.guardarCapa();
						}
					}finally{
						rv.stop();
					}
	
				} catch (Throwable e) {
					NotificationManager.addError(e);
					try {
						if(mbGuardar){
							loGuardar.cancelarCapa();
						}
					} catch (Throwable e1) {
						NotificationManager.addError(e1);
					}
				}finally{
					if (cancelMonitor != null) {
						cancelMonitor.setCurrentStep(cancelMonitor.getFinalStep());
						((DefaultCancellableMonitorable) cancelMonitor).setCanceled(true);
					}
					finished = true;
				}
			}

			public boolean isDefined() {
				return cancelMonitor.isDeterminatedProcess();
			}

			public boolean isCanceled() {
				return cancelMonitor.isCanceled();
			}

			public boolean isFinished() {
				return finished;
			}
		};
	}

	private DefaultCancellableMonitorable createCancelMonitor() {
		DefaultCancellableMonitorable monitor = new DefaultCancellableMonitorable();
		monitor.setInitialStep(0);
		monitor.setDeterminatedProcess(true);
		try {
			int totalSteps = 0;
			totalSteps = firstLayer.getSource().getShapeCount();
			monitor.setFinalStep(totalSteps);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return monitor;
	}

	public ILayerDefinition createLayerDefinition() {
		// TODO Auto-generated method stub
		return null;
	}

}

interface IDatosAGuardar {
	/**Devuelve el indice del campo, Crea el campo si no existe*/
	public int crearSeleccionarCampo(String psCampo) throws Throwable;
	/**Guarda un dato en la capa*/
	public void guardar(int plIndex, IGeometry geom, Value[] poValoresO, String psID, HashMap poValoresExtra) throws Throwable;
	/**Guarda la capa fisicamente*/
	public void guardarCapa() throws Throwable;
	/**Cancela cambios*/
	public void cancelarCapa() throws Throwable;
	
}
class JDatosAGuardarMismaCapa implements IDatosAGuardar {
	private FLyrVect firstLayer;
	private VectorialEditableAdapter moEdit=null;
	private boolean mbStart = false;
	private int mlCamposNuevos=0;
	private View moVista;
	private Throwable moError;
	
	public JDatosAGuardarMismaCapa(FLyrVect firstLayer, View poVista){
		this.firstLayer = firstLayer;
		moVista = poVista;
	}
	private VectorialEditableAdapter getEdit() throws EditionException{
		if(moEdit == null){
			if(!firstLayer.isEditing()){
				FLayer[] loActivas = moVista.getMapControl().getMapContext().getLayers().getActives();
				for(int i = 0 ;i < loActivas.length; i++){
					loActivas[i].setActive(false);
				}
				firstLayer.setActive(true);
				StartEditing loStart = new StartEditing();
				loStart.execute("");
				
				
			}
			moEdit = (VectorialEditableAdapter) firstLayer.getSource();
			
//			EditionManager editionManager=CADExtension.getEditionManager();
//			editionManager.setMapControl(moVista.getMapControl());
//			Legend legendOriginal=firstLayer.getLegend();
//			firstLayer.addLayerListener(editionManager);
//			firstLayer.setEditing(true);
//			moEdit = (VectorialEditableAdapter) firstLayer.getSource();
//			if (!(firstLayer.getSource().getDriver() instanceof IndexedShpDriver)){
//				VectorialLayerEdited vle=(VectorialLayerEdited)editionManager.getLayerEdited(firstLayer);
//				vle.setLegend(legendOriginal);
//			}
//			moEdit.getCommandRecord().addCommandListener(moVista.getMapControl());
//			StartEditing.startCommandsApplicable(moVista, firstLayer);
			
		}
		return moEdit;
	}

	public int crearSeleccionarCampo(String psCampo) throws Throwable{
		int lIndex = -1;
		//vemos si existe el campo
		lIndex = getEdit().getRecordset().getFieldIndexByName(psCampo);
		if(lIndex<0){
			//indice del campo nuevo
			lIndex = getEdit().getRecordset().getFieldCount();
			//decimos q hay un campo nuevo
			mlCamposNuevos++;
			//creamos el fielddescripcion
			final FieldDescription loFielDescrip = new FieldDescription();
			loFielDescrip.setFieldName(psCampo);
			loFielDescrip.setFieldDecimalCount(8);
			loFielDescrip.setFieldLength(50);
			loFielDescrip.setFieldType(Types.DOUBLE);
			loFielDescrip.setDefaultValue(
					ValueFactory.createValueByType("0.0", Types.DOUBLE)
					);
			//aadimos el field a la lista de campos
			SwingUtilities.invokeAndWait(
					new Runnable(){
						public void run() {
							try {
								getEdit().addField(loFielDescrip);
								moError=null;
							} catch (Throwable e) {
								moError=e;
							}
						}
					}
					);
			if(moError!=null){
				throw moError;
			}
		}
		return lIndex;
	}

	public void guardar(final int plIndex, final IGeometry geom, final Value[] poValoresO, final String psID, final HashMap poValoresExtra) throws Throwable{
		if(!mbStart){
			mbStart = true;
		}
		//creamos un array de los valores + los campos extra
		final Value[] loValores= new Value[poValoresO.length + mlCamposNuevos];
		for(int i = 0 ; i < poValoresO.length; i++){
			loValores[i] = poValoresO[i];
		}
		//establecemos los campos extra
		Object[] loKeys = poValoresExtra.keySet().toArray();
		for(int i = 0 ; i < loKeys.length; i++){
			Object loKey = loKeys[i];
			loValores[ ((Integer)loKey).intValue() ] = (Value) poValoresExtra.get(loKey);  
		}
		//modificamos la fila
		SwingUtilities.invokeAndWait(
				new Runnable(){
					public void run() {
						try {
							getEdit().modifyRow(
									plIndex, 
									new DefaultFeature(geom, loValores, psID), 
									"Editar valor", EditionEvent.ALPHANUMERIC);
							moError=null;
						} catch (Throwable e) {
							moError=e;
						}
					}
				}
				);
		if(moError!=null){
			throw moError;
		}
	}
	public void guardarCapa() throws Throwable {
		if(mbStart){
			StopEditing loStop = new StopEditing();
			loStop.executeSaveLayer(firstLayer);
//			firstLayer.setProperty("stoppingEditing",new Boolean(true));
//			try{
//
//				ISpatialWriter writer = (ISpatialWriter) getEdit().getWriter();
//				firstLayer.setRecordset(getEdit().getRecordset()); 
//				ILayerDefinition lyrDef = EditionUtilities.createLayerDefinition(firstLayer);
//				lyrDef.setShapeType(firstLayer.getShapeType());
//				writer.initialize(lyrDef);
//				getEdit().stopEdition(writer, EditionEvent.GRAPHIC);
//				firstLayer.setEditing(false);
//			}finally{
//				firstLayer.setProperty("stoppingEditing",new Boolean(false));
//			}
		}
	}
	public void cancelarCapa() throws Exception{
		if(mbStart){
			firstLayer.setProperty("stoppingEditing",new Boolean(true));
			com.iver.andami.ui.mdiManager.IWindow[] views = PluginServices
					.getMDIManager().getAllWindows();
			VectorialEditableAdapter vea = (VectorialEditableAdapter) firstLayer
					.getSource();
			vea.cancelEdition(EditionEvent.GRAPHIC);
			for (int j = 0; j < views.length; j++) {
				if (views[j] instanceof Table) {
					Table table = (Table) views[j];
					if (table.getModel().getAssociatedTable() != null
							&& table.getModel().getAssociatedTable().equals(firstLayer)) {
						table.cancelEditing();
					}
				}
			}
			firstLayer.setProperty("stoppingEditing",new Boolean(false));
			vea.getCommandRecord().removeCommandListener(moVista.getMapControl());
			firstLayer.setEditing(false);
			if (firstLayer.isSpatiallyIndexed())
	            {
	            	if(firstLayer.getISpatialIndex() != null)
	                {
	                	try {
							PluginServices.
								cancelableBackgroundExecution(new CreateSpatialIndexMonitorableTask((FLyrVect)firstLayer));
						} catch (DriverIOException e) {
							NotificationManager.addError(e);
						} catch (DriverException e) {
							NotificationManager.addError(e);
						}
	                }
	            }
			
		}
	}
}
class JDatosAGuardarCapaMemoria implements IDatosAGuardar {
	private FLyrVect firstLayer;
	private ConcreteMemoryDriver moEdit=null;
	private int mlCamposNuevos=0;
	private String msNombreCapa;
	
	public JDatosAGuardarCapaMemoria(FLyrVect firstLayer, String psNombre){
		this.firstLayer = firstLayer;
		msNombreCapa = psNombre;
	}
	private ConcreteMemoryDriver getEdit() throws Throwable{
		if(moEdit == null){
			ConcreteMemoryDriver driver = new ConcreteMemoryDriver();
			driver.setShapeType(firstLayer.getShapeType());
			driver.getTableModel().setColumnIdentifiers(
					firstLayer.getRecordset().getFieldNames()
					);
			moEdit = driver;
		}
		return moEdit;
	}

	public int crearSeleccionarCampo(String psCampo) throws Throwable{
		int lIndex = -1;
		//vemos si existe el campo
		lIndex = firstLayer.getRecordset().getFieldIndexByName(psCampo);
		if(lIndex<0){
			//indice del campo nuevo
			lIndex = firstLayer.getRecordset().getFieldCount()+mlCamposNuevos;
			//decimos q hay un campo nuevo
			mlCamposNuevos++;
			//aadimos el field a la lista de campos
			getEdit().getTableModel().addColumn(psCampo);
		}
		return lIndex;
	}

	public void guardar(int plIndex, IGeometry geom, Value[] poValoresO, String psID, HashMap poValoresExtra) throws Throwable{
		//creamos un array de los valores + los campos extra
		Value[] loValores= new Value[poValoresO.length + mlCamposNuevos];
		for(int i = 0 ; i < poValoresO.length; i++){
			loValores[i] = poValoresO[i];
		}
		//establecemos los campos extra
		Object[] loKeys = poValoresExtra.keySet().toArray();
		for(int i = 0 ; i < loKeys.length; i++){
			Object loKey = loKeys[i];
			loValores[ ((Integer)loKey).intValue() ] = (Value) poValoresExtra.get(loKey);  
		}
		//modificamos la fila
		getEdit().addGeometry(geom, loValores);
	}
	public void guardarCapa() throws Throwable {
		View v = (View) PluginServices.getMDIManager().getActiveWindow();
		MapControl mapCtrl = v.getMapControl();
		FLayer lyr = LayerFactory.createLayer(
				msNombreCapa,
				getEdit(), 
				mapCtrl.getProjection());
		if (lyr != null) {
			lyr.setVisible(true);
			mapCtrl.getMapContext().getLayers().addLayer(lyr);			
		}
	}
	public void cancelarCapa() throws Throwable{
	}
}

//private int crearSeleccionarCampo(String psCampo) throws DriverException, com.iver.cit.gvsig.fmap.DriverException, EditionException, ParseException{
//	int lIndex = -1;
//	FieldDescription loFielDescrip = new FieldDescription();
//	loFielDescrip.setFieldName(psCampo);
//	loFielDescrip.setFieldDecimalCount(2);
//	loFielDescrip.setFieldType(Types.DOUBLE);
//	loFielDescrip.setDefaultValue(
//			ValueFactory.createValueByType("0.0", Types.DOUBLE)
//			);			
//	if(resultLayerDefinition!=null){
//		FieldDescription[] loFields = resultLayerDefinition.getFieldsDesc();
//		for(int i = 0 ;i < loFields.length; i++){
//			if(loFields[i].equals(psCampo)){
//				lIndex = i;
//			}
//		}
//		if(lIndex<0){
//			FieldDescription[] loFieldsR = new FieldDescription[loFields.length+1];
//			for(int i = 0 ;i < loFields.length; i++){
//				loFieldsR[i] = loFields[i];
//			}
//			loFieldsR[loFields.length] = loFielDescrip;
//			resultLayerDefinition.setFieldsDesc(loFieldsR);
//		}
//		
//	}else{
//		lIndex = firstLayer.getRecordset().getFieldIndexByName(psCampo);
//		if(lIndex<0){
//			lIndex = firstLayer.getRecordset().getFieldCount();
//			ReadableVectorial rv=firstLayer.getSource();
//			EditableAdapter ea;
//			if (rv instanceof VectorialEditableAdapter){
//				ea=(EditableAdapter)firstLayer.getSource();
//			}else{
//				ea=new EditableAdapter();
//				SelectableDataSource sds=firstLayer.getRecordset();
//				ea.setOriginalDataSource(sds);
//			}
//			
//			ea.addField(loFielDescrip);
//		}
//	}
//	return lIndex;
//}


//private VectorialEditableAdapter getEdicion() throws Exception{
//	VectorialEditableAdapter ea=null;
//	if(resultLayerDefinition!=null){
//		ConcreteMemoryDriver driver = new ConcreteMemoryDriver();
//		driver.setShapeType(firstLayer.getShapeType());
//		
//		driver.getTableModel().addColumn(columnName)
//		
//		
//		resultProcessor = new FeaturePersisterProcessor(
//					resultLayerDefinition, schemaManager, 
//					writer);
//		resultProcessor.setSelectableDataSource(firstLayer.getRecordset());
//	}else{
//		firstLayer.setEditing(true);
//		ReadableVectorial rv=firstLayer.getSource();
//		
//		if (rv instanceof VectorialEditableAdapter){
//			ea=(VectorialEditableAdapter)firstLayer.getSource();
//		}else{
//			//TODO: Ojo falta
////			ea=new EditableAdapter();
////			SelectableDataSource sds=firstLayer.getRecordset();
////			ea.setOriginalDataSource(sds);
//		}
//	}
//	return ea;
//}
