/*
 * MultiVistaDocumentGUI.java
 *
 * Created on 10 de julio de 2008, 9:29
 */

package mn.multivista.documents.gui;

import com.iver.andami.ui.mdiManager.WindowInfo;
import com.iver.cit.gvsig.fmap.MapControl;
import com.iver.cit.gvsig.project.documents.gui.WindowData;
import com.iver.cit.gvsig.project.documents.view.IProjectView;
import com.iver.cit.gvsig.project.documents.view.MapOverview;
import com.iver.cit.gvsig.project.documents.view.gui.View;
import com.iver.cit.gvsig.project.documents.view.toc.gui.TOC;
import com.iver.utiles.console.JConsole;
import com.iver.utiles.console.ResponseListener;
import java.awt.BorderLayout;
import java.awt.image.BufferedImage;
import java.util.List;
import mn.multivista.documents.MultiVistaDocument;
import org.cresques.cts.IProjection;

/**
 *
 * @author  eduardo
 */
public class MultiVistaDocumentGUI extends View implements IMultivistaListener {
    
    protected WindowInfo m_viewInfo = null;
    private MultiVistaDocument moModelo;
    private MultiVistaGUI moMultiVista;
    private boolean mbInicializando = true;
    
    /** Creates new form MultiVistaDocumentGUI */
    public MultiVistaDocumentGUI() {
        mbInicializando = true;
        try{
            initComponents();
            moMultiVista = new MultiVistaGUI();
            moMultiVista.addListenerMultiVista(this);
            jPanelVistas.add(moMultiVista, BorderLayout.CENTER);
            
            cmbSincronizacion.removeAllItems();
            cmbSincronizacion.addItem("Centro y escala");
            cmbSincronizacion.addItem("Solo centro");
            cmbSincronizacion.addItem("Sin sincronizar");
            cmbSincronizacion.setSelectedIndex(moMultiVista.getTipoSincronizacion());
            
            chkVisualizarSelectorDeVistas.setSelected(moMultiVista.isVisualizarDespegablesVistas());
            
            
        }finally{
            mbInicializando = false;
        }
    }
    
    public void vistaCambiada(VistaSimpleYComboGUI poVistaYCombo) {
        if(!mbInicializando){
            rellenarTOC();
        }
    }
    
    public void setModel(final MultiVistaDocument pomodel) {
        moModelo=pomodel;
        moMultiVista.setModel(moModelo);
//        rellenarTOC();
    }
    
    /**
     * @see com.iver.mdiApp.ui.MDIManager.IWindow#windowActivated()
     */
    public void windowActivated() {
        moMultiVista.windowActivated();
    }
    
    public void windowClosed() {
        moMultiVista.windowClosed();
    }
    
    public boolean isPalette() {
        return moMultiVista.isPalette();
    }
    
    public void restore() {
        moMultiVista.restore();
    }
    
    public void toPalette() {
        moMultiVista.toPalette();
    }
    
    public Object getWindowModel() {
        return moModelo;
    }
    
    public WindowInfo getWindowInfo() {
        if (m_viewInfo == null) {
            m_viewInfo = new WindowInfo(WindowInfo.ICONIFIABLE | WindowInfo.RESIZABLE |
                    WindowInfo.MAXIMIZABLE);
            
            m_viewInfo.setWidth(750);
            m_viewInfo.setHeight(500);
            m_viewInfo.setTitle("MultiVista");
        }
        return m_viewInfo;
    }
    
    public MapControl getMapControl() {
        return moMultiVista.getVistaActiva().getVistaGUI().getMapControl();
    }
    
    public TOC getTOC() {
        return moMultiVista.getVistaActiva().getVistaGUI().getTOC();
    }
    
    
    public IProjectView getModel() {
        return moMultiVista.getVistaActiva().getVistaGUI().getModel();
    }
    
    public MapOverview getMapOverview() {
        return moMultiVista.getVistaActiva().getVistaGUI().getMapOverview();
    }
    
    public void repaintMap() {
        moMultiVista.repaintMap();
    }
    
    public BufferedImage getImage() {
        return moMultiVista.getVistaActiva().getVistaGUI().getMapControl().getImage();
    }
    
    public void setProjection(IProjection proj) {
        moMultiVista.setProjection(proj);
    }
    public IProjection getProjection() {
        return moMultiVista.getVistaActiva().getVistaGUI().getMapControl().getProjection();
    }
    
    public WindowData getWindowData() {
        updateWindowData();
        return windowData;
    }
    
    private void rellenarTOC() {
        List loListTOC = moMultiVista.getTOCS();
        jTabbedPane1.removeAll();
        jTabbedPane1.add("General", jPanelGeneral);
        
        for(int i = 0 ; i < loListTOC.size(); i++){
            TOC loToc = (TOC)loListTOC.get(i);
            jTabbedPane1.add("Vista " + String.valueOf(i),loToc);
        }
        
    }
    
    private void updateWindowData() {
        if (windowData == null)
            windowData = new WindowData();
        windowData.set("MainWindow.X", Integer.toString(this.getX()));
        windowData.set("MainWindow.Y", Integer.toString(this.getY()));
        windowData.set("MainWindow.Width", Integer.toString(this.getWidth()));
        windowData.set("MainWindow.Height", Integer.toString(this.getHeight()));
        windowData.set("Mutivista.Numero", cmbVistas.getSelectedItem().toString());
        windowData.set("Mutivista.TipoSincronizacion", Integer.toString(cmbSincronizacion.getSelectedIndex()));
        windowData.set("Mutivista.VisualizarSelecionVistas", Boolean.toString(chkVisualizarSelectorDeVistas.isSelected()));
        
        if (isPalette())
            windowData.set("GraphicLocator.isPalette", "true");
        else {
            windowData.set("GraphicLocator.isPalette", "false");
        }
        
        moMultiVista.updateWindowData(windowData);
        
    }
    
    public void setWindowData(WindowData data) {
        windowData = data;
    }
    
    public void focusConsole(String string) {
        super.focusConsole(string);
    }
    
    public void addConsoleListener(String string, ResponseListener responseListener) {
        super.addConsoleListener(string, responseListener);
    }
    
//    public void setModel(ProjectView projectView) {
//        super.setModel(projectView);
//    }
    
    public void removeConsoleListener(ResponseListener responseListener) {
        super.removeConsoleListener(responseListener);
    }
    
    public void showConsole() {
        super.showConsole();
    }
    
    public void initialize() {
        //super.initialize();
    }
    
    public void hideConsole() {
        super.hideConsole();
    }
    
    public JConsole getConsolePanel() {
        JConsole retValue;
        
        retValue = super.getConsolePanel();
        return retValue;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    protected void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel = new javax.swing.JPanel();
        jScrollPaneLAteral = new javax.swing.JScrollPane();
        jTabbedPane1 = new javax.swing.JTabbedPane();
        jPanelGeneral = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        cmbVistas = new javax.swing.JComboBox();
        jLabel4 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        cmbSincronizacion = new javax.swing.JComboBox();
        jLabel3 = new javax.swing.JLabel();
        chkVisualizarSelectorDeVistas = new javax.swing.JCheckBox();
        jLabel6 = new javax.swing.JLabel();
        btnRefrescarListasDeVistas = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jPanelVistas = new javax.swing.JPanel();

        setLayout(new java.awt.BorderLayout());

        jPanel.setLayout(new java.awt.BorderLayout());

        jPanelGeneral.setLayout(new java.awt.GridBagLayout());

        jLabel2.setText("NVistas");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(jLabel2, gridBagConstraints);

        cmbVistas.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "2", "3", "4", "6", "8", "9", "10", "12" }));
        cmbVistas.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cmbVistasItemStateChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(cmbVistas, gridBagConstraints);

        jLabel4.setText("  ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(jLabel4, gridBagConstraints);

        jLabel1.setText("Sincronizacin");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(jLabel1, gridBagConstraints);

        cmbSincronizacion.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Item 1", "Item 2", "Item 3", "Item 4" }));
        cmbSincronizacion.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cmbSincronizacionItemStateChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(cmbSincronizacion, gridBagConstraints);

        jLabel3.setText("  ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(jLabel3, gridBagConstraints);

        chkVisualizarSelectorDeVistas.setText("Visualizar seleccin de vistas");
        chkVisualizarSelectorDeVistas.setBorder(null);
        chkVisualizarSelectorDeVistas.setMargin(new java.awt.Insets(0, 0, 0, 0));
        chkVisualizarSelectorDeVistas.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                chkVisualizarSelectorDeVistasItemStateChanged(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(chkVisualizarSelectorDeVistas, gridBagConstraints);

        jLabel6.setText("  ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(jLabel6, gridBagConstraints);

        btnRefrescarListasDeVistas.setText("Refrescar despegables vistas ");
        btnRefrescarListasDeVistas.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnRefrescarListasDeVistasActionPerformed(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelGeneral.add(btnRefrescarListasDeVistas, gridBagConstraints);

        jLabel5.setText("  ");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weighty = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(2, 2, 0, 0);
        jPanelGeneral.add(jLabel5, gridBagConstraints);

        jTabbedPane1.addTab("General", jPanelGeneral);

        jScrollPaneLAteral.setViewportView(jTabbedPane1);

        jPanel.add(jScrollPaneLAteral, java.awt.BorderLayout.CENTER);

        jSplitPane1.setLeftComponent(jPanel);

        jPanelVistas.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        jPanelVistas.setLayout(new java.awt.BorderLayout());
        jSplitPane1.setRightComponent(jPanelVistas);

        add(jSplitPane1, java.awt.BorderLayout.CENTER);
    }// </editor-fold>//GEN-END:initComponents
    
    private void btnRefrescarListasDeVistasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnRefrescarListasDeVistasActionPerformed
        if(!mbInicializando){
            try{
                moMultiVista.refrescarDespegablesDeVistas();
            }catch(Exception e){
                moModelo.errorThrown(e);
            }
        }
    }//GEN-LAST:event_btnRefrescarListasDeVistasActionPerformed
    
    private void chkVisualizarSelectorDeVistasItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_chkVisualizarSelectorDeVistasItemStateChanged
        if(!mbInicializando){
            try{
                moMultiVista.setVisualizarDespegablesVistas(chkVisualizarSelectorDeVistas.isSelected());
            }catch(Exception e){
                moModelo.errorThrown(e);
            }
        }
        
    }//GEN-LAST:event_chkVisualizarSelectorDeVistasItemStateChanged
    
    private void cmbSincronizacionItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cmbSincronizacionItemStateChanged
        if(!mbInicializando){
            try{
                moMultiVista.setTipoSincronizacion(cmbSincronizacion.getSelectedIndex());
            }catch(Exception e){
                moModelo.errorThrown(e);
            }
        }
        
    }//GEN-LAST:event_cmbSincronizacionItemStateChanged
    
    private void cmbVistasItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_cmbVistasItemStateChanged
        if(!mbInicializando){
            try{
//                mbInicializando=true;
                moMultiVista.setNumeroVistas(Integer.valueOf(cmbVistas.getSelectedItem().toString()).intValue());
//                rellenarTOC();
//                mbInicializando=false;
            }catch(Exception e){
                moModelo.errorThrown(e);
            }finally{
//                mbInicializando=false;
            }
        }
        
    }//GEN-LAST:event_cmbVistasItemStateChanged
    
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnRefrescarListasDeVistas;
    private javax.swing.JCheckBox chkVisualizarSelectorDeVistas;
    private javax.swing.JComboBox cmbSincronizacion;
    private javax.swing.JComboBox cmbVistas;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JPanel jPanel;
    private javax.swing.JPanel jPanelGeneral;
    private javax.swing.JPanel jPanelVistas;
    private javax.swing.JScrollPane jScrollPaneLAteral;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTabbedPane jTabbedPane1;
    // End of variables declaration//GEN-END:variables
    
    
}
