package mn.vistaSuperpuesta;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import com.iver.cit.gvsig.fmap.MapControl;
import javax.swing.SwingUtilities;

public class JVistaSuperpuestaPintor {
    public static int mclPintorLupa = 0;
    public static int mclPintorMagnify = 1;
    
    private MapControl moMapControl;
    private JVistaSuperpuestaParam moParam;
    
    private int mlX;
    private int mlY;
    private boolean mbPintar=false;
    private boolean mbFinalizar=false;
    private boolean mbImagenesCargadas=false;
    
    private Image moImagenFuente;
    
    
    //proceso cargador
    
    private Thread moThread;
    
    
    private IVistaSuperpuestaDestino moDestino;
    
    
    
    public JVistaSuperpuestaPintor(
            JVistaSuperpuestaParam poParam,
            MapControl poMapControl, 
            JVistaSuperpuestaAccion poAccion) throws Exception{
        moParam = poParam;
        moMapControl = poMapControl;
        
        if(moParam.getTipoPintor() == mclPintorLupa){
            moDestino = new JVistaSuperpuestaPintorLupa();
        }else{
            moDestino = new JVistaSuperpuestaPintorMagnify();
        }
        moDestino.setDatos(moMapControl, moParam, poAccion);
        
        capturarImagenes();
        
    }
    /**Recarga las imagenes de la fuente y del mapa original*/
    public void capturarImagenes() throws Exception{
        moThread = new Thread(new Runnable(){
            public void run() {
                try {
                    capturarImagenesInterno();
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
            
        });
        moThread.start();
    }
    private void capturarImagenesInterno() throws Exception{
        synchronized (this){
            mbImagenesCargadas=false;
        }
        try{
            moDestino.capturarImagenes();
            capturarFuente();
        }finally{
            synchronized (this){
                mbImagenesCargadas=true;
            }
        }
    }
    
    private void capturarFuente() throws Exception{
        Rectangle2D loRect = moMapControl.getMapContext().getViewPort().getExtent();
        
        moImagenFuente = moParam.getFuente().getImage(
                moMapControl.getWidth() * moParam.getZoom(),
                moMapControl.getHeight() * moParam.getZoom(),
                loRect
                );
    }
    /**establecemos las coordenadas y habilitamos el proximo pintado*/
    public synchronized void setCoordenadas(int plX, int plY){
        mlX=plX;
        mlY=plY;
        mbPintar=true;
        notifyAll();
    }
    
    /**finalizamos*/
    public synchronized void terminar(){
        mbFinalizar=true;
        moDestino.terminar();
        notifyAll();
    }
    /**pintamos la fuente*/
    public synchronized boolean pintar(){
        
        if(mbPintar){
            mbPintar=false;
            //centramos el cuadro fuente en funcion del zoom
            int lFuenteX = mlX * moParam.getZoom() - (moParam.getWidth()/2);
            int lFuenteY = mlY * moParam.getZoom() - (moParam.getHeight()/2);
            int lFuenteWidth = moParam.getWidth();
            int lFuenteHeight = moParam.getHeight();
            if(lFuenteX<0){
//                lFuenteWidth +=lFuenteX;
                lFuenteX=0;
            }
            if(lFuenteY<0){
//                lFuenteHeight +=lFuenteY;
                lFuenteY=0;
            }
            if(lFuenteX>(moMapControl.getWidth() * moParam.getZoom())){
                lFuenteWidth -=(lFuenteX-(moMapControl.getWidth() * moParam.getZoom()));
                lFuenteX=(moMapControl.getWidth() * moParam.getZoom());
            }
            if(lFuenteY>(moMapControl.getHeight() * moParam.getZoom())){
                lFuenteHeight -=(lFuenteY-(moMapControl.getHeight() * moParam.getZoom()));
                lFuenteY=(moMapControl.getHeight() * moParam.getZoom());
            }
            
                        
            //centramos el cuadro mapa
            int lDestinoX = mlX - moParam.getWidth()/2;
            int lDestinoY = mlY - moParam.getHeight()/2;
            int lDestinoWidth = moParam.getWidth();
            int lDestinoHeight = moParam.getHeight();
            if(lDestinoX<0){
//                lDestinoWidth +=lDestinoX;
                lDestinoX=0;
            }
            if(lDestinoY<0){
//                lDestinoHeight +=lDestinoY;
                lDestinoY=0;
            }
            if(lDestinoX>(moMapControl.getWidth())){
            	lDestinoWidth -=(lFuenteX-(moMapControl.getWidth() ));
                lDestinoX=(moMapControl.getWidth());
            }
            if(lDestinoY>(moMapControl.getHeight())){
            	lDestinoHeight -=(lFuenteY-(moMapControl.getHeight()));
                lDestinoY=(moMapControl.getHeight());
            }
            //pintamos
            return moDestino.pintar(
                    moImagenFuente,
                    lDestinoX, lDestinoY,
                    lDestinoWidth, lDestinoHeight,
                    lFuenteX, lFuenteY,
                    lFuenteWidth, lFuenteHeight,
                    mbImagenesCargadas
                    );
            
        }
        //si llega aqui no se ha pintado
        return false;
    }
    
//    public void run() {
//        synchronized (this) {
//            while(!mbFinalizar){
//                if(mbPintar){
//                    pintar();
//                }else{
//                    try{
//                        wait(1000);
//                    }catch(Exception e){
//                        
//                    }
//                }
//            }
//        }
//    }
}

/**lupa encima del mapa*/
class JVistaSuperpuestaPintorLupa implements IVistaSuperpuestaDestino {
    
    private MapControl moMapControl;
    private JVistaSuperpuestaParam moParam;
    
//    private BufferedImage moImagenMapa;
    private BufferedImage moImagenDoubleBuffered;
    
    //x,y anterior
    private int mlXAux=-1;
    private int mlYAux=-1;
    private int mlWitdhAux;
    private int mlHeightAux;
    private boolean mbCargado=false;
    
    private AlphaComposite moTransparencia;
    
    public void setDatos(MapControl poMapControl, JVistaSuperpuestaParam poParam, JVistaSuperpuestaAccion poAccion){
        moMapControl=poMapControl;
        moParam=poParam;
    }
    // Este es el mtodo que permite realizar la modificacin del
    // canal alfa de la imagen
    private AlphaComposite creaComposite( float alfa ) {
        int tipo = AlphaComposite.SRC_OVER;
        
        return( AlphaComposite.getInstance(tipo,alfa) );
    }
    
    public void capturarImagenes() throws Exception {
    	mbCargado=false;
        moTransparencia = creaComposite((float)((100.0-(float)moParam.getTransparencia())/100.0));
//        capturarMapa();
        moImagenDoubleBuffered  = new BufferedImage(
                moMapControl.getWidth(),
                moMapControl.getHeight(),
                BufferedImage.TYPE_INT_ARGB
                );
        //double buffed, pintamos sobre una imagen y luego esa imagen la ponemos de golpe en el mapa
//        Graphics2D loG =  (Graphics2D)moImagenDoubleBuffered.getGraphics();
//        //pintamos el mapa original
//        loG.drawImage(moImagenMapa,
//                0, 0,
//                moMapControl.getWidth(),
//                moMapControl.getHeight(),
//                0, 0,
//                moMapControl.getWidth(),
//                moMapControl.getHeight(),
//                Color.white,
//                null);
        mbCargado=true;
    }
    
//    private void capturarMapa() throws Exception{
//        if(moImagenMapa==null ||
//           moMapControl.getWidth() != moImagenMapa.getHeight() ||
//           moMapControl.getHeight()!= moImagenMapa.getWidth()){ 
//	        moImagenMapa = new BufferedImage(
//	                moMapControl.getWidth(),
//	                moMapControl.getHeight(),
//	                BufferedImage.TYPE_INT_ARGB
//	                );
//        }
//        Graphics2D loGra = (Graphics2D)moImagenMapa.getGraphics();
//        loGra.setColor(Color.white);
//        loGra.drawRect(
//                0 , 0 ,
//                moMapControl.getWidth(),
//                moMapControl.getHeight());
//        
//        
//        moMapControl.paint(loGra);
//        //
//        //pintamos en la imagen
//        //
////        MapContext locontext = moMapControl.getMapContext();
////        locontext.draw(
////                moImagenMapa, loGra,
////                moMapControl.getMapContext().getScaleView());
//        
////        ImageIO.write(moImagenMapa, "jpg", new File("/tmp/mapa.jpg"));
//    }
//    
    public synchronized boolean pintar(
            Image poImagenFuente,
            int plXDestino, int plYDestino,
            int plWitdhDestino, int plHeightDestino,
            int plXFuente, int plYFuente,
            int plWitdhFuente, int plHeightFuente,
            boolean pbImagenesCargadas){
        if(mbCargado){
	        Graphics2D loG;
	        //double buffed, pintamos sobre una imagen y luego esa imagen la ponemos de golpe en el mapa
	        loG = (Graphics2D)moImagenDoubleBuffered.getGraphics();
            //restauramos la img. anterior
            moMapControl.paint(loG);

//            //pintamos el trozo de mapa original previo
//	        if(mlXAux>=0 && mlYAux>=0){
//	            moMapControl.getGraphics().drawImage(
//	            		moImagenDoubleBuffered,
//	                    mlXAux, mlYAux,
//	                    mlXAux + mlWitdhAux ,
//	                    mlYAux + mlHeightAux,
//	                    mlXAux, mlYAux,
//	                    mlXAux + mlWitdhAux,
//	                    mlYAux + mlHeightAux,
//	                    Color.white,
//	                    null);
//	        }

	        if(pbImagenesCargadas){
		        Composite compOriginal = loG.getComposite();
		        try{
		            loG.setComposite( moTransparencia );
		            //ponemos la imagen nueva
		            loG.drawImage(poImagenFuente,
		                plXDestino, plYDestino,
		                plXDestino + plWitdhDestino,
		                plYDestino + plHeightDestino,
		                plXFuente, plYFuente,
		                plXFuente + plWitdhFuente,
		                plYFuente + plHeightFuente,
		                Color.white,
		                null);
		        }finally{
		            loG.setComposite( compOriginal );
		        }
	        }
	        
	        //rectangulo de resaltado
	        loG.setColor(Color.black);
	        loG.drawRect(
	                plXDestino, plYDestino,
	                plWitdhDestino - 1 ,
	                plHeightDestino - 1);
	        if(!pbImagenesCargadas){
	        	loG.drawString(
	        			"cargando...", 
		                plXDestino, plYDestino+plWitdhDestino/2);
	        }
	        
	        //pintamos en el componente mapa
	        moMapControl.getGraphics().drawImage(
	                moImagenDoubleBuffered,
	                min(plXDestino, mlXAux),
	                min(plYDestino, mlYAux),
	                max(plXDestino+plWitdhDestino, mlXAux+mlWitdhAux),
	                max(plYDestino+plHeightDestino, mlYAux+mlHeightAux),
	                min(plXDestino, mlXAux),
	                min(plYDestino, mlYAux),
	                max(plXDestino+plWitdhDestino, mlXAux+mlWitdhAux),
	                max(plYDestino+plHeightDestino, mlYAux+mlHeightAux),
	                Color.white,
	                null
	                );
	        
	        //ponemos las ult. coor pintadas
	        mlXAux=plXDestino;
	        mlYAux=plYDestino;
	        mlWitdhAux=plWitdhDestino;
	        mlHeightAux=plHeightDestino;
        }
        //devuelve q se ha pintado
        return true;
    }


    private int min(int plX1, int plX2) {
    	int lResult = 0;
		if(plX1==0){
			lResult = plX2;
		}else  if(plX2==0){
			lResult = plX1;
		} else if(plX1>plX2){
			lResult = plX2;
		}else{
			lResult = plX1;
		}
		if(lResult <50){
			lResult = 0;
		}
		return lResult;
    }
    private int max(int plX1, int plX2) {
    	int lResult = 0;
		if(plX1==0){
			lResult = plX2;
		}else  if(plX2==0){
			lResult = plX1;
		} else if(plX1<plX2){
			lResult = plX2;
		}else{
			lResult = plX1;
		}
		if(lResult < 50){
			lResult = 0;
		}
		return lResult;
    }
	public void terminar() {
        //pintamos el trozo de mapa original previo
        if(moImagenDoubleBuffered!=null){
            Graphics2D loG;
            //double buffed, pintamos sobre una imagen y luego esa imagen la ponemos de golpe en el mapa
            loG = (Graphics2D)moImagenDoubleBuffered.getGraphics();
            //restauramos la img. anterior
            moMapControl.paint(loG);

            moMapControl.getGraphics().drawImage(
            		moImagenDoubleBuffered,
                    0, 0,
                    Color.white,
                    null);
        }
		
        moImagenDoubleBuffered=null;
//        moImagenMapa=null;
    }
    
    
}
class JVistaSuperpuestaPintorMagnify implements IVistaSuperpuestaDestino, WindowListener {
    private MapControl moMapControl;
    private JVistaSuperpuestaParam moParam;
    
    private JFormMagnify moForm;
    private JVistaSuperpuestaAccion moAccion;

    public JVistaSuperpuestaPintorMagnify(){
        moForm = new JFormMagnify(this);
        moForm.addWindowListener(this);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                moForm.setVisible(true);
            }
        }); 
    }
    
    public void setDatos(MapControl poMapControl, JVistaSuperpuestaParam poParam, JVistaSuperpuestaAccion poAccion) {
        moMapControl=poMapControl;
        moParam=poParam;
        moAccion=poAccion;

        moParam.setHeight(moForm.getHeightMapa());
        moParam.setWidth(moForm.getWidthMapa());
    }
    
    public void capturarImagenes() throws Exception {
    }
    
    public synchronized boolean pintar(
    		Image poImagenFuente, 
    		int plXDestino, int plYDestino, 
    		int plWitdhDestino, int plHeightDestino, 
    		int plXFuente, int plYFuente, 
    		int plWitdhFuente, int plHeightFuente,
    		boolean pbImagenesCargadas) {
        if(moForm!=null && pbImagenesCargadas){
            moParam.setHeight(moForm.getHeightMapa());
            moParam.setWidth(moForm.getWidthMapa());
            //pintamos en el componente mapa
            Graphics2D loG = (Graphics2D)moForm.getPanel().getGraphics();
            //ponemos la imagen nueva
            loG.drawImage(poImagenFuente,
                0, 0,
                plWitdhDestino, plHeightDestino,
                plXFuente, plYFuente,
                plXFuente + plWitdhFuente,
                plYFuente + plHeightFuente,
                Color.white,
                null);
        }
        
        return true;
    }

    public void terminar() {
        if(moForm!=null){
            moForm.removeWindowListener(this);
            moForm.dispose();
        }
        moForm=null;
        moAccion=null;
        moMapControl=null;
    }

   

    public void windowOpened(WindowEvent e) {
    }

    public void windowClosing(WindowEvent e) {
        if(moAccion!=null){
            moAccion.terminado();
        }
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
  
    }

    public void windowActivated(WindowEvent e) {
    
    }

    public void windowDeactivated(WindowEvent e) {
    
    }
    
}
